var % model has 28 endogenous varibales, which are needed to be solved recursively

// 20 theoretical model variables 

% 17 domestic theoretical model variables

c_t,    % consumption
y_t,    % output
i_t,    % domestic interest rate
q_t,    % real excnge rate
s_t,    % term of trade
pii_t,  % domestic inflation
piiH_t, % home-goods inflation
piiF_t, % imported-good inflation
psiF_t, % the law of one price gap
a_t,    % real net foreign asset
mc_t,   % marginal cost
e_t,    % nominal exchange rate
eg_t,   % preference shock (they are AR(1) shock process)
ea_t,   % productivity shock (they are AR(1) shock process)
eM_t,   % interest rate shock (they are AR(1) shock process)
ecp_t,  % cost-push shock (they are AR(1) shock process)
phi_t,  % risk premium shock (they are AR(1) shock process)


% 3 foreign theoretical model variables

pii_start,  % foreign inflation
y_start,    % foreign output
i_start,    % foreign interest rate

// 8 observable variables

% 5 domestic observable varibales

y_obs,      % obs output (demeaned real output per capita growth)
pii_obs,    % obs inflation (demeaned inflation rate)
i_obs,      % obs interest rate (demeaned interest rate)
q_obs,      % obs real exchange rate (demeaned real exchange rate growth)
s_obs,      % term of trade (demeaned term of trade)

% 3 foreign observarbale variables

pii_starobs, % foreign obs inflation (demeaned inflation rate)
y_starobs,   % foreign obs output (demeaned real foreign output growth)
i_starobs;   % foreign obs interest rate (demeaned interest rate)

varexo 
// 8 exogenous shock variables corresponding to  8 observable variables 

% 5 domestic shocks (they are AR(1) shock process)

eps_g,      % Preference shock process (demand shock) corresponding to Euler consumption in equa.1
eps_a,      % Productivity shock process (supply shock) corresponding to marginal cost in equa.7
eps_cp,     % Productivity shock process (supply shock)corresponding to importing firm in equa.8
eps_phi,    % Risk premium shock corresponding to uncover interest rate parity in equa. 10
eps_M,      % Policy shock process correspoding to interest rate in equa.12

% 3 foreign shock in foreign VAR ( they are i.i.d white noise process)

eps_pst,    % shock in VAR foreign inflation corresponding in equa. 18
eps_yst,    % shock in VAR foreign ouput corresponding in equa. 19
eps_ist;    % shock in VAR foreign interest rate corresponding in equa.20


parameters
sigma,      % the inverse of intertemporal elasticity of substitutions ( check in household' utility function)
phi,        % the inverse elasticities of intertemporal labor supply ( check in household' utility function)
thetaH,     % domestic Calvo paramter
thetaF,     % import Calvo parameter
eta,        % elasticiy substitution home and foreign goods
h,          % habit consumption
deltaH,     % domestic price indexation
deltaF,     % foreign price indexation (importing firm)
rhoi,       % Taylor rule, interest rate smoothing
psi_pi,     % Taylor rule, inflation rate
psi_y,      % Taylor rule, output
psi_e,      % Taylor rule, exchange rate growth
psi_dy,     % Taylor rule, output growth
rho_a,      % parameter in AR (1) Productivity or technology shock process in equa.14
rho_g,      % parameter in AR (1) preference shock in equa.13
rho_phi,    % parameter in AR (1) risk premium shock process in equa.16
rho_cp,     % parameter in AR (1) import cost-push shock process in equa.15

alpha,      % openness parameter
beta,       % discount factor
chi,        % risk premium elasticity 

%rho_M,      % parameter in AR (1) interest rate shock process in equa.17
% parameters in foreign VAR

// 6 parameters in VAR foreign inflation
app1,       % parameter of 1st lag inflation in VAR foreign inflation
app2,       % parameter of 2nd lag inflation in VAR foreign inflation
apy1,       % parameter of 1st lag ouput in VAR foreign inflation
apy2,       % parameter of 2nd lag ouput in VAR foreign inflation
api1,       % parameter of 1st lag interest rate in VAR foreign inflation
api2,       % parameter of 2nd lag interest rate in VAR foreign inflation

// 6 parameters in VAR foreign ouput
ayp1,       % parameter of 1st lag inflation in VAR foreign ouput
ayp2,       % parameter of 2nd lag inflation in VAR foreign ouput
ayy1,       % parameter of 1st lag output in VAR foreign ouput
ayy2,       % parameter of 2nd lag output in VAR foreign ouput
ayi1,       % parameter of 1st lag interest rate in VAR foreign ouput
ayi2,       % parameter of 2nd lag interest rate in VAR foreign ouput 

// 6 parameters in VAR foreign interest rate
aip1,       % parameter of 1st lag inflation in VAR foreign interest rate
aip2,       % parameter of 2nd lag inflation in VAR foreign interest rate
aiy1,       % parameter of 1st lag output in VAR foreign interest rate
aiy2,       % parameter of 2nd lag output in VAR foreign interest rate
aii1,       % parameter of 1st lag foreign interest rate in VAR foreign interest rate
aii2;       % parameter of 2nd lag foreign interest rate in VAR foreign interest rate

//  Calibrating parameters
beta = 0.99;    % Discount factor

model(linear);

%-------------BLOCK 1: THE DOMESTIC ECONOMY BLOCK WITH 17 EQUATIONS-----------------

// [1] - Euler Equation
(1+h)*c_t = c_t(+1)+ h*c_t(-1) - ((1-h)/sigma)*(i_t -pii_t + eg_t - eg_t(+1));

// [2] - Market Clearing
(1- alpha)*c_t = y_t - alpha*eta*(2-alpha)*s_t - alpha*eta*psiF_t - alpha*y_start;

// [3] - Law of One Price Gap which corresponding to equa.17
psiF_t = q_t - (1-alpha)*s_t;

// [4] - Terms of Trade, which corresponding to equ.16
s_t - s_t(-1) = piiH_t - piiH_t;

// [5] - Real Exchange Rate which corresponding to equa.17
q_t = e_t + pii_start - pii_t;

// [6] - Domestic Price Inflation which corresponding to equa.18
(1 + beta*deltaH)*piiH_t = deltaH*piiH_t(-1) + ((1-thetaH)*(1-thetaH*beta)/thetaH)*mc_t + beta*piiH_t(+1) ;
 
// [7] - marginal cost
mc_t = phi*y_t + (1+ phi)*ea_t + alpha*s_t + (sigma/(1 - h))*(c_t - h*c_t(-1));

// [8] - Import Price Inflation which corresponding to equa.19
(1 + beta*deltaF)*piiF_t = deltaF*piiF_t(-1) + beta*piiF_t(+1) + ((1 - thetaF)*(1 - thetaF*beta)/thetaF)*psiF_t + ecp_t;

// [9] - Domestic CPI Inflation which corresponding to equa. 20
pii_t = piiH_t + alpha*(s_t - s_t(-1));

// [10] - Uncovered Interest Parity which corresponding to equa. 21
(i_t -pii_t(+1)) -(i_start - pii_start(+1)) = (q_t(+1) - q_t) - chi*a_t - phi_t;

// [11] - Foreign Asset Budget Constraint which corresponding to equa.22
c_t + a_t = (1/beta)* a_t(-1) - alpha*(s_t +psiF_t) + y_t;

// [12] - Taylor rule coresponding to equa.23
i_t = rhoi*i_t(-1) + (1-rhoi)*(psi_pi*pii_t + psi_y*y_t + psi_dy*(y_t - y_t(-1)) + psi_e*(e_t -e_t(-1))) + eM_t;
%i_t = rhoi*i_t(-1) + psi_pi*pii_t + psi_y*y_t + psi_dy*(y_t - y_t(-1)) + psi_e*(e_t -e_t(-1)) + eM_t;

%%%%%%%%%%--------------THE 5-AR DOMESTIC SHOCK PROCESSES------------------

// [13] Preference shock process (demand shock)
eg_t = rho_g*eg_t(-1) + eps_g;          % corresponding to Euler consumption in equa.1

// [14] Productivity shock process (supply shock)
ea_t = rho_a*ea_t(-1)+ eps_a; % corresponding to marginal cost in equa.7

// [15] Cost-push shock process (supply shock)
ecp_t = rho_cp*ecp_t(-1) + eps_cp;      % corresponding to importing firm in equa.8 

// [16] risk premium shock
phi_t = rho_phi*phi_t(-1) + eps_phi;    % corresponding to uncover interest rate parity in equa. 10

// [17] policy shock process 
eM_t = eps_M;                           % i.i.d process as JP(2010) assumption
%eM_t = rho_M*eM_t(-1) + eps_M;         % correspoding to interest rate in equa.12

%----------------BLOCK 2: FOREIGN ECONOMY WIH THE SECOND-ORDER VAR SYSTEM OF 3 EQUATIONS-----
 
// [18] VAR foreign inflation
pii_start = app1*pii_start(-1) + app2*pii_start(-2) + apy1*y_start(-1) + apy2*y_start(-2) + api1*i_start(-1) +api2*i_start(-2) + eps_pst;

// [19] VAR foreign output
y_start = ayp1*pii_start(-1) + ayp2*pii_start(-2) + ayy1*y_start(-1) + ayy2*y_start(-2) + ayi1*i_start(-1) +ayi2*i_start(-2) + eps_yst;

// [20] VAR foreign interest rate
i_start = aip1*pii_start(-1) + aip2*pii_start(-2) + aiy1*y_start(-1) + aiy2*y_start(-2) + aii1*i_start(-1) +aii2*i_start(-2) + eps_ist;

%-----------------BLOCK 3: THE 8-EQUATION MEASUREMENT SYSTEM-----------------------

// [21] Matching demostically theoretical output with obs. ouput
y_obs = y_t - y_t(-1);              % Note: y_obs is a demeaned output growth

// [22] Matching demostically theoretical inflation with obs. inflation
pii_obs = pii_t;                    % pii_obs is a demeaned inflation rate

// [23] Matching demostically theoretical interest rate with obs. interest rate
i_obs = i_t;                        % i_obs is a demeaned interest rate

// [24] Matching demostically theoretical real exchage rate with obs. real exchange rate
q_obs = q_t - q_t(-1);              % Note: q_obs is a demeaned real exchange rate growth

// [25] Matching demostically theoretical term of trade with obs. term of trade
s_obs = s_t - s_t(-1);              % Note: s_obs is a demeaned term of trade growth

// [26] Matching foreign theoretical ouput with obs. foreign output
y_starobs = y_start - y_start(-1) ; % Note: y_starobs  is a demeaned foreign output growth

// [27] Matching foreign theoretical inflation with obs. foreign inflation
pii_starobs = pii_start;            % pii_starobs is a demeaned foreign inflation rate

// [28] Matching foreign theoretical interest rate with obs. foreign interest rate
i_starobs = i_start;                % i_starobs is a demeaned foreign interest rate

end;

steady;
check;

estimated_params; // estimating 43 parameters, in which 25 main DSGE parameters and 18 foreign VAR parameters

% [1] the inverse of intertemporal elasticity of substitutions ( check in household' utility function)
sigma, GAMMA_PDF, 1.2,0.4;     

% [2] the inverse elasticities of intertemporal labor supply ( check in household' utility function)
phi,  GAMMA_PDF,1.50,0.75;      

% [3]  domestic Calvo paramter
thetaH, BETA_PDF, 0.5, 0.1;    

% [4] import Calvo parameter
thetaF, BETA_PDF, 0.5, 0.1;       

% [5] elasticiy substitution home and foreign goods
eta, GAMMA_PDF, 1.50,0.75;        

% [6]  habit consumption
h, BETA_PDF, 0.5, 0.25;           

% [7] domestic price indexation
deltaH, BETA_PDF, 0.5, 0.25;       

% [8] foreign price indexation (importing firm)
deltaF, BETA_PDF, 0.5, 0.25;   
     
% [9] Taylor rule, interest rate smoothing
rhoi,  BETA_PDF, 0.5, 0.25;         

% [10] Taylor rule, inflation rate
psi_pi,  GAMMA_PDF, 1.50,0.3;    

% [11] Taylor rule, output
psi_y,    GAMMA_PDF,0.25,0.13;   

% [12] Taylor rule, exchange rate growth
psi_e,    GAMMA_PDF,0.25,0.13;    

% [13] Taylor rule, output growth 
psi_dy,   GAMMA_PDF,0.25,0.13;     

% [14] parameter in AR (1) Productivity or technology shock process in equa.14
rho_a, BETA_PDF, 0.80, 0.10;
     
% [15] parameter in AR (1) preference shock in equa.13
rho_g,BETA_PDF, 0.80, 0.10;
      
% [16] parameter in AR (1) risk premium shock process in equa.16
rho_phi, BETA_PDF, 0.80, 0.10;
   
% [17]  parameter in AR (1) import cost-push shock process in equa.15
rho_cp, BETA_PDF, 0.50, 0.25;
    
% [18] sd foreign inflation in foreign VAR in equa.18
stderr eps_pst, INV_GAMMA_PDF,0.5, inf;
 
% [19] sd foreign output in foreign VAR in equa.19
stderr eps_yst, INV_GAMMA_PDF,0.5, inf;

% [20] sd foreign interest rate in foreign VAR in equa.20
stderr eps_ist, INV_GAMMA_PDF,0.5, inf;

% [21] sd technology
stderr eps_a, INV_GAMMA_PDF,0.5, inf;

% [22] sd taylor rule
stderr eps_M, INV_GAMMA_PDF,0.5, inf;

% [23] sd preference
stderr eps_g, INV_GAMMA_PDF,0.5, inf;

% [24] sd risk premium
stderr eps_phi, INV_GAMMA_PDF,0.5, inf;

% [25] sd import cost-push
stderr eps_cp, INV_GAMMA_PDF,0.5, inf;

// 6 parameters in VAR foreign inflation
% [26] parameter of 1st lag inflation in VAR foreign inflation
app1, NORMAL_PDF, 0.59, 0.2; 

% [27]  parameter of 2nd lag inflation in VAR foreign inflation   
app2,  NORMAL_PDF, 0, 0.25;     

% [28] parameter of 1st lag ouput in VAR foreign inflation
apy1, NORMAL_PDF,0, 0.3;      

% [29] parameter of 2nd lag ouput in VAR foreign inflation
apy2, NORMAL_PDF, 0, 0.15;      

% [30] parameter of 1st lag interest rate in VAR foreign inflation
api1, NORMAL_PDF,0, 0.3;   

% [31] parameter of 2nd lag interest rate in VAR foreign inflation  
api2, NORMAL_PDF, 0,0.15;      

// 6 parameters in VAR foreign ouput
% [32] parameter of 1st lag inflation in VAR foreign ouput
ayp1, NORMAL_PDF,0, 0.3; 
      
% [33] parameter of 2nd lag inflation in VAR foreign ouput
ayp2,   NORMAL_PDF, 0,0.15;    

% [34] parameter of 1st lag output in VAR foreign ouput
ayy1, NORMAL_PDF, 0.9, 0.10;   
   
% [35] parameter of 2nd lag output in VAR foreign ouput
ayy2, NORMAL_PDF, 0, 0.25;   
    
% [36] parameter of 1st lag interest rate in VAR foreign ouput
ayi1, NORMAL_PDF, 0, 0.3;
     
% [37] parameter of 2nd lag interest rate in VAR foreign ouput 
ayi2,   NORMAL_PDF, 0,0.15;    

// 6 parameters in VAR foreign interest rate
% [38] parameter of 1st lag inflation in VAR foreign interest rate
aip1,  NORMAL_PDF,0, 0.3; 
      
% [39] parameter of 2nd lag inflation in VAR foreign interest rate
aip2, NORMAL_PDF, 0,0.15;
      
% [40] parameter of 1st lag output in VAR foreign interest rate
aiy1,  NORMAL_PDF, 0, 0.3; 
      
% [41] parameter of 2nd lag output in VAR foreign interest rate
aiy2,  NORMAL_PDF, 0,0.15;
      
% [42] parameter of 1st lag foreign interest rate in VAR foreign interest rate
aii1,NORMAL_PDF, 0.9, 0.10;
       
% [43] parameter of 2nd lag foreign interest rate in VAR foreign interest rate
aii2, NORMAL_PDF, 0, 0.25;  
     
end; 

// 8 observable variables corresponding to 8 shocks in model
varobs y_obs pii_obs i_obs q_obs s_obs y_starobs pii_starobs i_starobs; 

estimation(
datafile= Data_for_Dynare,
diffuse_filter,
plot_priors =1,
mode_compute =9,
//mode_file=,
mode_check,

mh_nblocks=1,
mh_replic=500000,
mh_drop=0.45,
mh_jscale=0.25
//mh_init_scale=,
 );